@@ -8,9 +8,11 @@ of stack usage.
 
 import xdis.opcodes.opcode_3x as opcode_3x
 from xdis.opcodes.base import (
-    finalize_opcodes, init_opdata,
+    finalize_opcodes, format_extended_arg, init_opdata,
     update_pj3)
 
+from xdis.opcodes.opcode_3x import format_MAKE_FUNCTION_arg
+
 # FIXME: can we DRY this even more?
 
 version = 3.2
@@ -24,4 +26,9 @@ init_opdata(l, opcode_3x, version)
 
 update_pj3(globals(), l)
 
+opcode_arg_fmt = {
+    'MAKE_FUNCTION': format_MAKE_FUNCTION_arg,
+    'EXTENDED_ARG': format_extended_arg,
+}
+
 finalize_opcodes(l)
