@@ -3,7 +3,7 @@ Extracted from Python 3 dis module but generalized to
 allow running on Python 2.
 """
 
-import sys, types
+import re, sys, types
 from xdis import PYTHON3
 
 from collections import namedtuple
@@ -19,28 +19,38 @@ else:
 
 _have_code = (types.MethodType, types.FunctionType, types.CodeType, type)
 
+def extended_arg_val(opc, val):
+    return val << opc.EXTENDED_ARG_SHIFT
 
-def pretty_format_value_flags(flags):
-    if (flags & 0x03) == 0x00:
-        return ''
-    elif (flags & 0x03) == 0x01:
-        return '!s'
-    elif (flags & 0x03) == 0x02:
-        return '!r'
-    elif (flags & 0x03) == 0x03:
-        return '!a'
-    elif (flags & 0x04) == 0x04:
-        # pop fmt_spec from the stack and use it, else use an
-        # empty fmt_spec.
-        return ''
+# This is modified from Python 3.6's dis
+def unpack_opargs_bytecode(code, opc):
+    extended_arg = 0
+    try:
+        n = len(code)
+    except TypeError:
+        code = code.co_code
+        n = len(code)
 
-def findlinestarts(code):
+    offset = 0
+    while offset < n:
+        prev_offset = offset
+        op = code2num(code, offset)
+        offset += 1
+        if op_has_argument(op, opc):
+            arg = code2num(code, offset) | extended_arg
+            extended_arg = extended_arg_val(opc, arg) if op == opc.EXTENDED_ARG else 0
+            offset += 2
+        else:
+            arg = None
+        yield (prev_offset, op, arg)
+
+def findlinestarts(code, dup_lines=False):
     """Find the offsets in a byte code which are start of lines in the source.
 
     Generate pairs (offset, lineno) as described in Python/compile.c.
 
     """
-    if PYTHON3:
+    if not isinstance(code.co_lnotab, str):
         byte_increments = list(code.co_lnotab[0::2])
         line_increments = list(code.co_lnotab[1::2])
     else:
@@ -52,12 +62,16 @@ def findlinestarts(code):
     addr = 0
     for byte_incr, line_incr in zip(byte_increments, line_increments):
         if byte_incr:
-            if lineno != lastlineno:
+            if (lineno != lastlineno or
+                (dup_lines and 0 < byte_incr < 255)):
                 yield (addr, lineno)
                 lastlineno = lineno
+                pass
             addr += byte_incr
+            pass
         lineno += line_incr
-    if lineno != lastlineno:
+    if (lineno != lastlineno or
+        (dup_lines and 0 < byte_incr < 255)):
         yield (addr, lineno)
 
 def offset2line(offset, linestarts):
@@ -86,36 +100,56 @@ def offset2line(offset, linestarts):
         return linestarts[len(linestarts)-1][1]
     return linestarts[high][1]
 
-def findlabels(code, opc):
-    """Detect all offsets in a byte code which are jump targets.
-
-    Return the list of offsets.
-
+def get_jump_targets(code, opc):
+    """Returns a list of instruction offsets in the supplied bytecode
+    which are the targets of some sort of jump instruction.
     """
-    labels = []
-    # enumerate() is not an option, since we sometimes process
-    # multiple elements on a single pass through the loop
-    try:
-        n = len(code)
-    except:
-        code = code.co_code
-        n = len(code)
-    i = 0
-    while i < n:
-        op = code2num(code, i)
-        i = i+1
-        if op >= opc.HAVE_ARGUMENT:
-            arg = code2num(code, i) + code2num(code, i+1)*256
-            i = i+2
-            label = -1
-            if op in opc.hasjrel:
-                label = i+arg
-            elif op in opc.hasjabs:
-                label = arg
-            if label >= 0:
-                if label not in labels:
-                    labels.append(label)
-    return labels
+    offsets = []
+    for offset, op, arg in unpack_opargs_bytecode(code, opc):
+        if arg is not None:
+            jump_offset = -1
+            if op in opc.JREL_OPS:
+                jump_offset = offset + 3 + arg
+            elif op in opc.JABS_OPS:
+                jump_offset = arg
+            if jump_offset >= 0:
+                if jump_offset not in offsets:
+                    offsets.append(jump_offset)
+    return offsets
+
+def get_jump_target_maps(code, opc):
+    """Returns a dictionary where the key is an offset and the values are
+    a list of instruction offsets which can get run before that
+    instruction. This includes jump instructions as well as non-jump
+    instructions. Therefore, the keys of the dictionary are reachible
+    instructions. The values of the dictionary may be useful in control-flow
+    analysis.
+    """
+    offset2prev = {}
+    prev_offset = -1
+    for offset, op, arg in unpack_opargs_bytecode(code, opc):
+        if prev_offset >= 0:
+            prev_list = offset2prev.get(offset, [])
+            prev_list.append(prev_offset)
+            offset2prev[offset] = prev_list
+        if op in opc.NOFOLLOW:
+            prev_offset = -1
+        else:
+            prev_offset = offset
+        if arg is not None:
+            jump_offset = -1
+            if op in opc.JREL_OPS:
+                jump_offset = offset + 3 + arg
+            elif op in opc.JABS_OPS:
+                jump_offset = arg
+            if jump_offset >= 0:
+                prev_list = offset2prev.get(jump_offset, [])
+                prev_list.append(offset)
+                offset2prev[jump_offset] = prev_list
+    return offset2prev
+
+
+findlabels = get_jump_targets
 
 def _get_const_info(const_index, const_list):
     """Helper to get optional details about const references
@@ -128,6 +162,11 @@ def _get_const_info(const_index, const_list):
     if const_list is not None:
         argval = const_list[const_index]
 
+    # float values nan and inf are not directly representable in Python at least
+    # before 3.5 and even there it is via a library constant.
+    # So we will canonicalize their representation as float('nan') and float('inf')
+    if isinstance(argval, float) and str(argval) in frozenset(['nan', '-nan', 'inf', '-inf']):
+        return argval, "float('%s')" % argval
     return argval, repr(argval)
 
 def _get_name_info(name_index, name_list):
@@ -148,7 +187,7 @@ def _get_name_info(name_index, name_list):
         argrepr = repr(argval)
     return argval, argrepr
 
-def get_instructions_bytes(code, opc, varnames=None, names=None, constants=None,
+def get_instructions_bytes(bytecode, opc, varnames=None, names=None, constants=None,
                            cells=None, linestarts=None, line_offset=0):
     """Iterate over the instructions in a bytecode string.
 
@@ -158,7 +197,7 @@ def get_instructions_bytes(code, opc, varnames=None, names=None, constants=None,
     arguments.
 
     """
-    labels = findlabels(code, opc)
+    labels = opc.findlabels(bytecode, opc)
     extended_arg = 0
 
     # FIXME: We really need to distinguish 3.6.0a1 from 3.6.a3.
@@ -168,11 +207,11 @@ def get_instructions_bytes(code, opc, varnames=None, names=None, constants=None,
     starts_line = None
     # enumerate() is not an option, since we sometimes process
     # multiple elements on a single pass through the loop
-    n = len(code)
+    n = len(bytecode)
     i = 0
     extended_arg = 0
     while i < n:
-        op = code2num(code, i)
+        op = code2num(bytecode, i)
 
         offset = i
         if linestarts is not None:
@@ -185,57 +224,91 @@ def get_instructions_bytes(code, opc, varnames=None, names=None, constants=None,
         argval = None
         argrepr = ''
         has_arg = op_has_argument(op, opc)
+        optype = None
         if has_arg:
             if python_36:
-                arg = code2num(code, i) | extended_arg
-                extended_arg = (arg << 8) if opc == opc.EXTENDED_ARG else 0
+                arg = code2num(bytecode, i) | extended_arg
+                extended_arg = (arg << 8) if op == opc.EXTENDED_ARG else 0
                 # FIXME: Python 3.6.0a1 is 2, for 3.6.a3 we have 1
                 i += 1
             else:
-                arg = code2num(code, i) + code2num(code, i+1)*256 + extended_arg
+                arg = code2num(bytecode, i) + code2num(bytecode, i+1)*256 + extended_arg
                 i += 2
                 extended_arg = arg*65536 if op == opc.EXTENDED_ARG else 0
 
             #  Set argval to the dereferenced value of the argument when
             #  availabe, and argrepr to the string representation of argval.
-            #    _disassemble_bytes needs the string repr of the
+            #    disassemble_bytes needs the string repr of the
             #    raw name index for LOAD_GLOBAL, LOAD_CONST, etc.
             argval = arg
-            if op in opc.hasconst:
+            if op in opc.CONST_OPS:
                 argval, argrepr = _get_const_info(arg, constants)
-            elif op in opc.hasname:
+                optype = 'const'
+            elif op in opc.NAME_OPS:
                 argval, argrepr = _get_name_info(arg, names)
-            elif op in opc.hasjrel:
+                optype = 'name'
+            elif op in opc.JREL_OPS:
                 argval = i + arg
                 argrepr = "to " + repr(argval)
-            elif op in opc.hasjabs:
+                optype = 'jrel'
+            elif op in opc.JABS_OPS:
                 argval = arg
                 argrepr = "to " + repr(argval)
-            elif op in opc.haslocal:
+                optype = 'jabs'
+            elif op in opc.LOCAL_OPS:
                 argval, argrepr = _get_name_info(arg, varnames)
-            elif op in opc.hascompare:
+                optype = 'local'
+            elif op in opc.COMPARE_OPS:
                 argval = opc.cmp_op[arg]
                 argrepr = argval
-            elif op in opc.hasfree:
+                optype = 'compare'
+            elif op in opc.FREE_OPS:
                 argval, argrepr = _get_name_info(arg, cells)
-            elif op in opc.hasnargs:
+                optype = 'free'
+            elif op in opc.NARGS_OPS:
+                optype = 'nargs'
                 if not python_36:
                     argrepr = ("%d positional, %d keyword pair" %
-                               (code2num(code, i-2), code2num(code, i-1)))
-            elif python_36 and op == opc.FORMAT_VALUE:
-                argrepr = pretty_format_value_flags(arg)
+                               (code2num(bytecode, i-2), code2num(bytecode, i-1)))
+            # This has to come after hasnargs. Some are in both?
+            elif op in opc.VARGS_OPS:
+                optype = 'vargs'
+            if hasattr(opc, 'opcode_arg_fmt') and opc.opname[op] in opc.opcode_arg_fmt:
+                argrepr = opc.opcode_arg_fmt[opc.opname[op]](arg)
         elif python_36:
             i += 1
 
         opname = opc.opname[op]
-        yield Instruction(opname, op, arg, argval, argrepr,
+        inst_size = op_size(op, opc)
+        yield Instruction(opname, op, optype, inst_size, arg, argval, argrepr,
                           has_arg, offset, starts_line, is_jump_target)
 
 def op_has_argument(op, opc):
     return op >= opc.HAVE_ARGUMENT
 
+def next_offset(op, opc, offset):
+    return offset + instruction_size(op, opc)
+
+# FIXME: this would better be called an instr_size
+# since it is about instructions, not opcodes
+def instruction_size(op, opc):
+    """For a given opcode, `op`, in opcode module `opc`,
+    return the size, in bytes, of an `op` instruction.
+
+    This is the size of the opcode (1 byte) and any operand it has. In
+    Python before version 3.6 this will be either 1 or 3 bytes.  In
+    Python 3.6 or later, it is 2 bytes or a "word"."""
+    if op < opc.HAVE_ARGUMENT:
+        return 2 if opc.version >= 3.6 else 1
+    else:
+        return 2 if opc.version >= 3.6 else 3
+
+# Compatiblity
+op_size = instruction_size
+
+
 _Instruction = namedtuple("_Instruction",
-     "opname opcode arg argval argrepr has_arg offset starts_line is_jump_target")
+     "opname opcode optype inst_size arg argval argrepr has_arg offset starts_line is_jump_target")
 
 class Instruction(_Instruction):
     """Details for a bytecode operation
@@ -243,55 +316,94 @@ class Instruction(_Instruction):
        Defined fields:
          opname - human readable name for operation
          opcode - numeric code for operation
+         optype - opcode classification. One of
+            compare, const, free, jabs, jrel, local, name, nargs
+         inst_size - number of bytes the instruction occupies
          arg - numeric argument to operation (if any), otherwise None
          argval - resolved arg value (if known), otherwise same as arg
          argrepr - human readable description of operation argument
          has_arg - True opcode takes an argument. In that case,
                    argval and argepr will have that value. False
                    if this opcode doesn't take an argument. In that case,
-                   don't look at argval or ardrepr.
+                   don't look at argval or argrepr.
          offset - start index of operation within bytecode sequence
          starts_line - line started by this opcode (if any), otherwise None
          is_jump_target - True if other code jumps to here, otherwise False
     """
+    # FIXME: remove has_arg from initialization but keep it as a field.
 
-    def _disassemble(self, lineno_width=3, mark_as_current=False):
+    def disassemble(self, lineno_width=3, mark_as_current=False, asm_format=False):
         """Format instruction details for inclusion in disassembly output
 
         *lineno_width* sets the width of the line number field (0 omits it)
         *mark_as_current* inserts a '-->' marker arrow as part of the line
         """
         fields = []
+        if asm_format:
+            indexed_operand = set(['name', 'local', 'compare', 'free'])
         # Column: Source code line number
         if lineno_width:
             if self.starts_line is not None:
-                lineno_fmt = "%%%dd" % lineno_width
-                fields.append(lineno_fmt % self.starts_line)
+                if asm_format:
+                    lineno_fmt = "%%%dd:\n" % lineno_width
+                    fields.append(lineno_fmt % self.starts_line)
+                    fields.append(' ' * (lineno_width))
+                    if self.is_jump_target:
+                        fields.append(' ' * (lineno_width-1))
+                else:
+                    lineno_fmt = "%%%dd:" % lineno_width
+                    fields.append(lineno_fmt % self.starts_line)
             else:
-                fields.append(' ' * lineno_width)
+                fields.append(' ' * (lineno_width+1))
         # Column: Current instruction indicator
-        if mark_as_current:
+        if mark_as_current and not asm_format:
             fields.append('-->')
         else:
             fields.append('   ')
         # Column: Jump target marker
         if self.is_jump_target:
-            fields.append('>>')
+            if not asm_format:
+                fields.append('>>')
+            else:
+                fields = ["L%d:\n" % self.offset] + fields
+                if not self.starts_line:
+                    fields.append(' ')
         else:
             fields.append('  ')
         # Column: Instruction offset from start of code sequence
-        fields.append(repr(self.offset).rjust(4))
+        if not asm_format:
+            fields.append(repr(self.offset).rjust(4))
+
         # Column: Opcode name
         fields.append(self.opname.ljust(20))
         # Column: Opcode argument
         if self.arg is not None:
-            fields.append(repr(self.arg).rjust(5))
+            argrepr = self.argrepr
+            if asm_format:
+                if self.optype == 'jabs':
+                    fields.append('L' + str(self.arg))
+                elif self.optype == 'jrel':
+                    argval = self.offset + self.arg + self.inst_size
+                    fields.append('L' + str(argval))
+                elif self.optype in indexed_operand:
+                    fields.append('(%s)' % argrepr)
+                    argrepr = None
+                elif (self.optype == 'const'
+                      and not re.search('\s', argrepr)):
+                    fields.append('(%s)' % argrepr)
+                    argrepr = None
+                else:
+                    fields.append(repr(self.arg))
+            else:
+                fields.append(repr(self.arg).rjust(6))
             # Column: Opcode argument details
-            if self.argrepr:
-                fields.append('(' + self.argrepr + ')')
+            if argrepr:
+                fields.append('(%s)' % argrepr)
+                pass
+            pass
         return ' '.join(fields).rstrip()
 
-    # FIXME: figure out how to do _disassemble passing in opnames
+    # FIXME: figure out how to do disassemble passing in opnames
 
 class Bytecode(object):
     """The bytecode operations of a piece of code
@@ -301,7 +413,8 @@ class Bytecode(object):
 
     Iterating over this yields the bytecode operations as Instruction instances.
     """
-    def __init__(self, x, opc, first_line=None, current_offset=None):
+    def __init__(self, x, opc, first_line=None, current_offset=None,
+                 dup_lines=False):
         self.codeobj = co = get_code_object(x)
         if first_line is None:
             self.first_line = co.co_firstlineno
@@ -310,7 +423,7 @@ class Bytecode(object):
             self.first_line = first_line
             self._line_offset = first_line - co.co_firstlineno
         self._cell_names = co.co_cellvars + co.co_freevars
-        self._linestarts = dict(opc.findlinestarts(co))
+        self._linestarts = dict(opc.findlinestarts(co, dup_lines=dup_lines))
         self._original_object = x
         self.opc = opc
         self.opnames = opc.opname
@@ -319,9 +432,9 @@ class Bytecode(object):
     def __iter__(self):
         co = self.codeobj
         return get_instructions_bytes(co.co_code, self.opc, co.co_varnames, co.co_names,
-                                       co.co_consts, self._cell_names,
-                                       self._linestarts,
-                                       line_offset=self._line_offset)
+                                      co.co_consts, self._cell_names,
+                                      self._linestarts,
+                                      line_offset=self._line_offset)
 
     def __repr__(self):
         return "{}({!r})".format(self.__class__.__name__,
@@ -338,7 +451,7 @@ class Bytecode(object):
         """Return formatted information about the code object."""
         return format_code_info(self.codeobj, self.opc.version)
 
-    def dis(self):
+    def dis(self, asm_format=False):
         """Return a formatted view of the bytecode operations."""
         co = self.codeobj
         if self.current_offset is not None:
@@ -352,12 +465,14 @@ class Bytecode(object):
                                linestarts=self._linestarts,
                                line_offset=self._line_offset,
                                file=output,
-                               lasti=offset)
+                               lasti=offset,
+                               asm_format=asm_format)
         return output.getvalue()
 
     def disassemble_bytes(self, code, lasti=-1, varnames=None, names=None,
                           constants=None, cells=None, linestarts=None,
-                          file=sys.stdout, line_offset=0):
+                          file=sys.stdout, line_offset=0,
+                          asm_format=False):
         # Omit the line number column entirely if we have no line number info
         show_lineno = linestarts is not None
         # TODO?: Adjust width upwards if max(linestarts.values()) >= 1000?
@@ -371,7 +486,8 @@ class Bytecode(object):
             if new_source_line:
                 file.write("\n")
             is_current_instr = instr.offset == lasti
-            file.write(instr._disassemble(lineno_width, is_current_instr) + "\n")
+            file.write(instr.disassemble(lineno_width, is_current_instr, asm_format)
+                       + "\n")
             pass
         return
 
@@ -414,7 +530,7 @@ def list2bytecode(l, opc, varnames, consts):
         print(opname, operands)
         gen = (j for j in operands if operands)
         for j in gen:
-            k = (consts if opcode in opc.hasconst else varnames).index(j)
+            k = (consts if opcode in opc.CONST_OPS else varnames).index(j)
             if k == -1:
                 raise TypeError(
                     "operand %s [%s, %s], not found in names" %
@@ -436,6 +552,7 @@ def list2bytecode(l, opc, varnames, consts):
 if __name__ == '__main__':
     import xdis.opcodes.opcode_27  as opcode_27
     import xdis.opcodes.opcode_34  as opcode_34
+    import xdis.opcodes.opcode_36  as opcode_36
     consts = (None, 2)
     varnames = ('a')
     instructions = [
@@ -456,3 +573,5 @@ if __name__ == '__main__':
     print(bc)
     bc = list2bytecode(instructions, opcode_34, varnames, consts)
     print(bc)
+    bc = list2bytecode(instructions, opcode_36, varnames, consts)
+    print(bc)
