@@ -1,3 +1,4 @@
+# (C) Copyright 2017 by Rocky Bernstein
 """
 CPython 2.0 bytecode opcodes
 
@@ -6,7 +7,7 @@ This is similar to the opcode portion in Python 2.0's dis.py library.
 
 import xdis.opcodes.opcode_21 as opcode_21
 from xdis.opcodes.base import (
-    init_opdata, finalize_opcodes, rm_op)
+    init_opdata, finalize_opcodes, format_extended_arg, rm_op, update_pj2)
 
 version = 2.0
 
@@ -20,11 +21,10 @@ rm_op(l, 'LOAD_CLOSURE',  135)
 rm_op(l, 'LOAD_DEREF',    136)
 rm_op(l, 'STORE_DEREF',   137)
 
-# FIXME remove (fix uncompyle6)
-def updateGlobal():
-    globals().update({'PJIF': l['opmap']['JUMP_IF_FALSE']})
-    globals().update({'PJIT': l['opmap']['JUMP_IF_TRUE']})
-    return
+update_pj2(globals(), l)
+
+opcode_arg_fmt = {
+    'EXTENDED_ARG': format_extended_arg,
+}
 
-updateGlobal()
 finalize_opcodes(l)
