@@ -8,17 +8,18 @@ of stack usage.
 
 import xdis.opcodes.opcode_2x as opcode_2x
 from xdis.opcodes.base import (
-    init_opdata, finalize_opcodes)
+    finalize_opcodes, format_extended_arg,
+    init_opdata, update_pj2)
 
 version = 2.3
 
 l = locals()
 init_opdata(l, opcode_2x, version)
 
-# FIXME remove (fix uncompyle6)
-def updateGlobal():
-    globals().update({'PJIF': l['opmap']['JUMP_IF_FALSE']})
-    globals().update({'PJIT': l['opmap']['JUMP_IF_TRUE']})
-    return
-updateGlobal()
+update_pj2(globals(), l)
+
+opcode_arg_fmt = {
+    'EXTENDED_ARG': format_extended_arg,
+}
+
 finalize_opcodes(l)
